#!/usr/bin/env python3
# coding: utf-8

import subprocess
import sys
import traceback

# Two problems to solve
# - play audio fully in terminal
# - play video in fullscreen mode


_cli_players = {
    "audio": [
        {
            "test": ["play", "--version"],
            "mark": "SoX",
        },
        {
            "test": ["mpv.com", "--version"],
            "mark": "mplayer",
        },
        {
            "test": ["mpv", "--version"],
            "mark": "mplayer",
        },
        {
            "test": ["ffplay", "-version"],
            "play": ["ffplay", "-nodisp"],
            "mark": "ffmpeg",
        },
        {
            "test": ["/Applications/VLC.app/Contents/MacOS/VLC", "--version"],
            "play": ["/Applications/VLC.app/Contents/MacOS/VLC", "-I", "rc"],
            "mark": "VLC",
        },
    ],
    "video": [
        {
            "test": ["mpv", "--version"],
            "play": ["mpv", "--fs", "--no-input-default-bindings", "--no-config"],
            "mark": "mplayer",
        },
        {
            "test": ["/Applications/VLC.app/Contents/MacOS/VLC", "--version"],
            "play": ["/Applications/VLC.app/Contents/MacOS/VLC", "-f"],
            "mark": "VLC",
        },
        {
            "test": ["ffplay", "-version"],
            "play": ["ffplay", "-fs"],
            "mark": "ffmpeg",
        },
    ],
}

_available_cli_players = {}


class PlayerNotFoundError(Exception):
    pass


def _find_cli_player(typ):
    for plr in _cli_players[typ]:
        print("debug: typ", typ)
        print("debug: plr", plr)
        try:
            sp = subprocess.run(plr["test"], stdout=subprocess.PIPE)
        except FileNotFoundError:
            continue
        except Exception:
            traceback.print_exc()
            continue
        if sp.returncode:
            continue
        mark = plr["mark"]
        outtext = sp.stdout.decode()
        # treat lower-case mark as case-insensitive
        if mark.islower():
            outtext = outtext.lower()
        if mark in outtext:
            return plr
    raise PlayerNotFoundError


def get_cli_player(typ):
    global _available_cli_players
    try:
        return _available_cli_players[typ]
    except KeyError:
        return _available_cli_players.setdefault(typ, _find_cli_player(typ))


def _runsub(*args, **kwargs):
    try:
        subprocess.run(*args, **kwargs)
    except KeyboardInterrupt:
        print("\nKeyboardInterrupt\n", file=sys.stderr)


def _play(path, typ):
    plr = get_cli_player(typ)
    args = plr.get("play") or plr["test"][:1]
    args.append(path)
    _runsub(args)


def play_audio(path):
    _play(path, "audio")


def play_video(path):
    _play(path, "video")



#!/usr/bin/env python3
# coding: utf-8

import argparse
import pathlib

from joker.studio.aux import utils
from joker.studio.aux.info import MediaInfo
from joker.studio.ffmpeg.filters import vf_crop


def mkcod_crop(path, outpath, head, tail, *margins):
    xinfo = MediaInfo(path)
    width = xinfo.video.width
    height = xinfo.video.height
    duration = xinfo.get_video_duration()

    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("ss", head),
            ("t", duration - head - tail),
            ("c:a", "copy"),
        ]
    )
    if margins:
        cod["vf"] = vf_crop(width, height, *margins)
    else:
        cod["c:v"] = "copy"
    return cod("ffmpeg", outpath)


def run(prog=None, args=None):
    desc = "Rename files"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    utils.add_dry_option(parser)

    parser.add_argument(
        "-c",
        "--crop",
        type=float,
        nargs=4,
        metavar=("top", "right", "bottom", "left"),
        help="crop video in image dimensions",
    )

    parser.add_argument(
        "-t",
        "--trim",
        type=float,
        nargs=2,
        metavar=("head", "tail"),
        help="trim media in time dimension",
    )

    parser.add_argument("paths", metavar="PATH", nargs="+", help="files")
    ns = parser.parse_args(args)
    head, tail = ns.trim or (0, 0)
    for path in ns.paths:
        px = pathlib.Path(path)
        outpath = px.with_suffix(".crop" + px.suffix)
        cod = mkcod_crop(path, outpath, head, tail, *ns.crop)
        cod.run(ns.dry)


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# coding: utf-8

import argparse
import pathlib

from joker.studio.aux import utils
from joker.studio.aux.info import MediaInfo
from joker.studio.aux.utils import CommandOptionDict
from joker.studio.ffmpeg.filters import af_fade, vf_fade


def mkcod_fade(path, outpath, vfadein=0, vfadeout=0, afadein=0, afadeout=0):
    xinfo = MediaInfo(path)
    duration = xinfo.get_audio_duration()
    cod = CommandOptionDict(
        [
            ("i", path),
        ]
    )

    if vfadein or vfadeout:
        cod["vf"] = vf_fade(0, duration, vfadein, vfadeout)
    else:
        cod["c:v"] = "copy"

    if afadein or afadeout:
        cod["af"] = af_fade(0, duration, afadein, afadeout)
    else:
        cod["c:a"] = "copy"

    return cod("ffmpeg", outpath)


def run(prog=None, args=None):
    desc = "Add fade-in and fade-out to audios"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    utils.add_dry_option(parser)

    parser.add_argument(
        "-a",
        type=int,
        nargs=2,
        metavar=("IN", "OUT"),
        help="audio fade-in and fade-out lengths, in sec",
    )

    parser.add_argument(
        "-v",
        type=int,
        nargs=2,
        metavar=("IN", "OUT"),
        help="video fade-in and fade-out lengths, in sec",
    )

    parser.add_argument("paths", metavar="PATH", nargs="+", help="input video files")

    ns = parser.parse_args(args)
    for path in ns.paths:
        px = pathlib.Path(path)
        outpath = px.with_suffix(".fade" + px.suffix)
        vfi, vfo = ns.v or (0, 0)
        afi, afo = ns.a or (0, 0)
        cod = mkcod_fade(path, outpath, vfi, vfo, afi, afo)
        cod.run(ns.dry)


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# coding: utf-8

import os
import re
import subprocess
import tempfile

import numpy as np
from scipy.io import wavfile


def read_audio(path):
    if path.endswith(".wav"):
        return wavfile.read(path)
    tmp = tempfile.mkstemp()[1]
    cmd = ["ffmpeg", "-y", "-i", path, "-f", "wav", tmp]
    try:
        subprocess.run(cmd)
        return wavfile.read(tmp)
    finally:
        os.remove(tmp)


def chop(arr, winsize):
    assert arr.ndim == 1
    n = len(arr)
    arr = arr[: n - n % winsize]
    arr.shape = -1, winsize
    return arr.mean(axis=1)


def compute_audio_energy_string(path):
    if path.endswith(".wav"):
        sample_rate, data = wavfile.read(path)
    else:
        sample_rate, data = read_audio(path)

    # merge 2 channels, and limit value to exp(10)-1
    data = np.abs(data).max(axis=1)

    # 0.1 second pieces
    data = chop(data, int(sample_rate * 0.1))
    data = np.log2(1.0 + data) / 1.501
    data = data.astype(int)
    return "".join(format(i, "X") for i in data)


class AudioEnergySeries(object):
    def __init__(self, levels, winsize):
        self._levels = np.array(levels, dtype="uint8")
        self._string = "".join(format(i, "X") for i in self._levels)
        self._winsize = winsize

    def __str__(self):
        return self._string

    @classmethod
    def from_file(cls, path, winsize=0.1):
        sample_rate, data = read_audio(path)

        # merge 2 channels, and limit value to exp(10)-1
        data = np.abs(data).max(axis=1)

        # 0.1 second pieces
        data = chop(data, int(sample_rate * 0.1))
        data = np.log2(1.0 + data) / 1.501
        return cls(data, winsize)

    def find_silences(self, thresh=3, duration=1):
        assert 0 <= thresh <= 9
        n = round(duration / self._winsize)
        pat = r"[0-%d]{%d,}" % (thresh, n)
        for mat in re.finditer(pat, self._string):
            yield sum(mat.span()) / 2.0 * self._winsize



#!/usr/bin/env python3
# coding: utf-8

import argparse
import os
import pathlib

from joker.cast.syntax import printerr

from joker.studio.aux import utils


def mkcod_video_poster(path, outpath, pos):
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("ss", pos),
            ("vframes", 1),
        ]
    )
    return cod("ffmpeg", outpath)


def mkcod_pipe_thumbnail(path, pos, w, h):
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("ss", pos),
            ("vframes", 1),
            ("vcodec", "png"),
            ("f", "image2pipe"),
            ("s", "{}x{}".format(w, h)),
        ]
    )
    return cod("ffmpeg", "-")


def mkcod_video_thumbnail(path, outpath, tspan, count=None, size=None):
    fps = 1 / float(tspan)
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("vf", "fps={}".format(fps)),
        ]
    )
    if count is not None:
        cod["vframes"] = count
    if size is not None:
        cod["s"] = "{}x{}".format(*size)
    if outpath == "-":
        cod["vcodec"] = "png"
        cod["f"] = "image2pipe"
    elif count != 1 and "%" not in str(outpath):
        pm, ext = os.path.splitext(outpath)
        outpath = pm + ".Thumb_%04d" + ext
    return cod("ffmpeg", outpath)


def make_poster(path, ns):
    bx = os.path.splitext(path)
    outpath = bx[0] + "." + ns.format
    cod = mkcod_video_poster(path, outpath, 1)
    cod.run(ns.dry)


def make_thumbnails(path, ns):
    ext = ("." + ns.format).replace("..", ".")
    px = pathlib.Path(path)
    outpath = px.with_suffix(ext)
    cod = mkcod_video_thumbnail(path, outpath, ns.tspan)
    cod.run(ns.dry)


def poster(prog=None, args=None):
    desc = "generate poster images from videos"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    parser.add_argument(
        "-f",
        "--format",
        dest="fmt",
        choices=["png", "jpg"],
        default="jpg",
        help="out image format",
    )

    parser.add_argument(
        "--dry", action="store_true", help="print ffmpeg command but do not execute it"
    )

    parser.add_argument("paths", metavar="PATH", nargs="+", help="an audio/video file")
    ns = parser.parse_args(args)
    for p in ns.paths:
        try:
            make_poster(p, ns)
        except Exception as e:
            printerr("path:", p)
            printerr(e)


def thumb(prog=None, args=None):
    desc = "Generate thumbnail images from a video"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    # TODO: ExtendedArgumentParser?
    utils.add_dry_option(parser)

    parser.add_argument(
        "-t",
        "--tspan",
        type=int,
        default=60,
        help="time span between each image, in second",
    )

    parser.add_argument("-f", "--format", default="jpg", help="output image format")

    parser.add_argument("-l", "--label", help="output file label")

    parser.add_argument("paths", metavar="PATH", nargs="+", help="an video file")

    ns = parser.parse_args(args)
    for p in ns.paths:
        make_thumbnails(p, ns)



#!/usr/bin/env python3
# coding: utf-8


class MediaInfo(object):
    def __init__(self, path):
        # pymediainfo is slow to import, 600ms+
        import pymediainfo

        self.path = str(path)
        self.minfo = pymediainfo.MediaInfo.parse(path)

    def find_tracks(self, track_type):
        tt = track_type.lower()
        tracks = self.minfo.tracks
        return [tr for tr in tracks if tr.track_type.lower() == tt]

    def get_track(self, track_type):
        tracks = self.find_tracks(track_type)
        if not tracks:
            raise ValueError("no {} track found".format(track_type))
        if len(tracks) > 1:
            raise ValueError("multiple {} tracks found".format(track_type))
        return tracks[0]

    @property
    def video(self):
        return self.get_track("video")

    @property
    def audio(self):
        return self.get_track("audio")

    @property
    def image(self):
        return self.get_track("image")

    @property
    def general(self):
        return self.get_track("general")

    def get_audio_duration(self):
        return float(self.audio.duration) / 1000

    def get_video_duration(self):
        return float(self.video.duration) / 1000

    def get_size(self):
        tracks = self.find_tracks("Video") + self.find_tracks("Image")
        if not tracks:
            return None, None
        tr = tracks[0]
        return tr.width, tr.height

    def get_duration(self):
        tracks = self.find_tracks("Video") + self.find_tracks("Audio")
        if not tracks:
            return None
        tr = tracks[0]
        return float(tr.duration) / 1000



#!/usr/bin/env python3
# coding: utf-8

from volkanic.cmdline import CommandRegistry

cmddef = """\
avatar      joker.studio.misc.avatar
conv        joker.studio.ffmpeg.conv
crop        joker.studio.ffmpeg.crop
fade        joker.studio.ffmpeg.fade
ico         joker.studio.misc.avatar:mkico
imb         joker.studio.misc.avatar:mkimb
imt         joker.studio.misc.margin
iwh         joker.studio.misc.avatar:report_wh_ratios
poster      joker.studio.ffmpeg.thumb:poster
ren         joker.studio.misc.rename
rotate      joker.studio.images.rotate:main
split       joker.studio.ffmpeg.split
sub         joker.studio.ffmpeg.subtitle
thumb       joker.studio.ffmpeg.thumb:thumb
vid         joker.studio.misc.vident
"""

_prog = "python3 -m joker.sudio"
registry = CommandRegistry.from_cmddef(cmddef, _prog)

if __name__ == "__main__":
    registry()



#!/usr/bin/env python3
# coding: utf-8

import pathlib
import sys

from PIL import Image
from joker.cast.iterative import chunkwize_split


class AvatarMaker(object):
    def __init__(self, size=320, ext=".jpg", stretch=False):
        self.stretch = stretch
        self.size = size
        self.ext = ext

    def convert_image(self, path):
        im = Image.open(path).convert("RGB")
        if not self.stretch:
            w, h = im.size
            el = min(im.size)
            box = [
                int((w - el) / 2.0),
                int((h - el) / 2.0),
                int((w + el) / 2.0),
                int((h + el) / 2.0),
            ]
            im = im.crop(box)
        im = im.resize((self.size, self.size), Image.ANTIALIAS)

        px = pathlib.Path(path)
        prefix = "avatar-{0}x{0}.".format(self.size)
        name = prefix + px.stem + self.ext
        outpath = str(px.with_name(name))
        print(outpath, file=sys.stderr)
        im.save(outpath)

    def batch_convert(self, paths):
        for p in paths:
            self.convert_image(p)


def run(prog=None, args=None):
    import argparse

    desc = "make avatar"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    parser.add_argument(
        "-f",
        "--format",
        choices=["jpg", "png"],
        default="jpg",
        help="output image _format",
    )
    parser.add_argument(
        "-s", "--size", type=int, default=320, help="size of output image"
    )
    parser.add_argument(
        "-x",
        "--stretch",
        action="store_true",
        help="stretch image to a square (no crop)",
    )
    parser.add_argument(
        "filenames", metavar="FILENAME", nargs="+", help="path to an image file"
    )
    ns = parser.parse_args(args)
    ext = "." + ns.format
    AvatarMaker(ns.size, ext, ns.stretch).batch_convert(ns.filenames)


def convert_to_ico(path):
    px = pathlib.Path(path).with_suffix(".ico")
    img = Image.open(path)
    img.save(str(px))


def mkico(prog=None, args=None):
    import argparse

    desc = "convert images to ico _format (multiple sizes in 1 file)"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    parser.add_argument(
        "filenames", metavar="FILENAME", nargs="+", help="path to images"
    )
    ns = parser.parse_args(args)
    for path in ns.filenames:
        convert_to_ico(path)


def b64enc(path):
    from base64 import b64encode

    return b64encode(open(path, "rb").read())


class SmallImage(object):
    def __init__(self, binstr, fmt):
        fmt = fmt.lower()
        self._binstr = binstr
        self._format = {"jpg": "jpeg"}.get(fmt, fmt)

    @classmethod
    def from_file(cls, path, fmt=None):
        binstr = open(path, "rb").read()
        fmt = fmt or path.split(".")[-1]
        return cls(binstr, fmt)

    def b64encode(self):
        from base64 import b64encode

        return b64encode(self._binstr)

    def b64s(self):
        from base64 import b64encode

        return b64encode(self._binstr).decode()

    def html_tag(self):
        return '<img src="data:image/png;base64, {}" alt=""/>'.format(self.b64s())

    def pycode(self, width=60):
        pieces = chunkwize_split(width, self.b64encode())
        notations = (repr(s) for s in pieces)
        prefix = "_imb = "
        sep = " \\\n" + " " * len(prefix)
        parts = [
            prefix + sep.join(notations),
            "import io, base64; from PIL import Image",
            "icon = Image.open(io.BytesIO(base64.b64decode(_imb)))",
        ]
        return "\n\n".join(parts)


def mkimb(prog=None, args=None):
    import argparse

    desc = "Convert an image to base64 encoded bytes"
    parser = argparse.ArgumentParser(prog=prog, description=desc)

    parser.add_argument(
        "-s",
        "--style",
        choices=["py", "html"],
        help="print as python code or html img tag",
    )

    parser.add_argument(
        "-w",
        "--width",
        type=int,
        default=60,
        help="line width (0 for unlimited, N/A when style=html)",
    )

    parser.add_argument("filename", metavar="FILENAME", help="path to a small image")

    ns = parser.parse_args(args)
    si = SmallImage.from_file(ns.filename)
    if not ns.style:
        parts = chunkwize_split(ns.width, si.b64s())
        print("\n".join(parts))
    elif ns.style == "py":
        print(si.pycode(ns.width))
    elif ns.style == "html":
        print(si.html_tag())


def _average(nums):
    return sum(nums) / len(nums)


def report_wh_ratios(_, args: list):
    from joker.studio.aux.info import MediaInfo

    ratios = []
    widths = []
    heights = []
    for path in args:
        xinfo = MediaInfo(path)
        w = xinfo.image.width
        h = xinfo.image.height
        r = w / h
        print("{:.03f}".format(r), w, h, path)
        ratios.append(r)
        widths.append(w)
        heights.append(h)
    avg_r = _average(ratios)
    avg_w = int(_average(widths))
    avg_h = int(_average(heights))
    print("{:.03f}".format(avg_r), avg_w, avg_h, "<avg>")



#!/usr/bin/env python3
# coding: utf-8

import argparse
import pathlib
import sys

from joker.cast.syntax import printerr

from joker.studio.aux import utils


def mkcod_convert(path, outpath):
    cod = utils.CommandOptionDict([("i", path)])
    return cod("ffmpeg", outpath)


def mkcod_image_convert(path, outpath):
    cmd = "convert"
    if sys.platform.startswith("win"):
        cmd = "imagemagick"
    cod = utils.CommandOptionDict()
    return cod(cmd, path, outpath)


def _mkcod_convert_ts_mp4(path, outpath):
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("c:v", "libx264"),
            ("c:a", "copy"),
            ("bsf:a", "aac_adtstoasc"),
        ]
    )
    return cod("ffmpeg", outpath)


def _mkcod_convert_gif_mp4(path, outpath):
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("movflags", "faststart"),
            ("pix_fmt", "yuv420p"),
            ("vf", "scale=trunc(iw/2)*2:trunc(ih/2)*2"),
            ("c:v", "libx264"),
            ("an", []),
        ]
    )
    return cod("ffmpeg", outpath)


def mkcod_convert_to_mp4(path, outpath):
    if path.endswith(".ts"):
        return _mkcod_convert_ts_mp4(path, outpath)
    if path.endswith(".gif"):
        return _mkcod_convert_gif_mp4(path, outpath)
    cod = utils.CommandOptionDict([("i", path)])
    return cod("ffmpeg", outpath)


def mkcod_convert_to_audio(path, outpath):
    cod = utils.CommandOptionDict([("i", path)])
    if outpath.endswith(".mp3"):
        cod["c:a"] = "libmp3lame"
    return cod("ffmpeg", outpath)


MEDIATYPE_AUDIO = 1
MEDIATYPE_VIDEO = 2
MEDIATYPE_IMAGE = 3


known_extensions = {
    ".mp3": MEDIATYPE_AUDIO,
    ".wav": MEDIATYPE_AUDIO,
    ".aac": MEDIATYPE_AUDIO,
    ".ogg": MEDIATYPE_AUDIO,
    ".wma": MEDIATYPE_AUDIO,
    ".jpg": MEDIATYPE_IMAGE,
    ".bmp": MEDIATYPE_IMAGE,
    ".png": MEDIATYPE_IMAGE,
    ".gif": MEDIATYPE_IMAGE,
    ".mp4": MEDIATYPE_VIDEO,
    ".mkv": MEDIATYPE_VIDEO,
    ".flv": MEDIATYPE_VIDEO,
    ".wmv": MEDIATYPE_VIDEO,
    ".mov": MEDIATYPE_VIDEO,
    ".ts": MEDIATYPE_VIDEO,
    ".flac": MEDIATYPE_AUDIO,
    ".rmvb": MEDIATYPE_VIDEO,
    ".webm": MEDIATYPE_VIDEO,
}


class UnsupportedConversion(ValueError):
    pass


def check_conversion(inext, outext):
    inmt = known_extensions.get(inext)
    outmt = known_extensions.get(outext)
    if inmt and outmt and inmt == outmt:
        return inmt, outmt
    if inmt == MEDIATYPE_VIDEO and outmt == MEDIATYPE_AUDIO:
        return inmt, outmt
    msg = "{} => {}".format(inext, outext)
    raise UnsupportedConversion(msg)


def convert_a_file(path, ns):
    px = pathlib.Path(path)
    outext = ("." + ns.fmt).replace("..", ".")

    # cat for category
    incat, outcat = check_conversion(px.suffix, outext)
    if path.endswith(outext):
        outpath = str(px.with_suffix(".conv" + outext))
    else:
        outpath = str(px.with_suffix(outext))

    if outext == ".mp4":
        cod = mkcod_convert_to_mp4(path, outpath)
    elif outcat == MEDIATYPE_AUDIO:
        cod = mkcod_convert_to_audio(path, outpath)
    elif incat == MEDIATYPE_IMAGE and outcat == MEDIATYPE_IMAGE:
        cod = mkcod_image_convert(path, outpath)
    else:
        cod = mkcod_convert(path, outpath)
    cod.run(ns.dry)


def run(prog=None, args=None):
    desc = "convert audio/video format"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    utils.add_dry_option(parser)

    parser.add_argument(
        "-f",
        "--format",
        dest="fmt",
        default="mp4",
        help="out audio/video format, e.g. mp4, ogg, ...",
    )

    parser.add_argument("paths", metavar="PATH", nargs="+", help="an audio/video file")

    ns = parser.parse_args(args)
    for p in ns.paths:
        try:
            convert_a_file(p, ns)
        except Exception as e:
            printerr("path:", p)
            printerr(e)


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# coding: utf-8
import argparse
import pathlib

import numpy
from PIL import Image
from joker.cast.syntax import printerr

STDMAX = 3


def get_corner_size(arr):
    n = sum(arr.shape[:2]) / 8.0
    return int(n**0.5)


class MarginDetection(object):
    def __init__(self, img, stdmax=STDMAX):
        self.img = img.convert("RGB")
        self.arr = numpy.asarray(self.img).astype("uint32")
        self.maxstd = stdmax
        n = get_corner_size(self.arr)
        _sli_head = slice(None, n)
        _sli_tail = slice(-n, None)
        corner_pixels = numpy.stack(
            (
                self.arr[_sli_head, _sli_head, :],
                self.arr[_sli_head, _sli_tail, :],
                self.arr[_sli_tail, _sli_tail, :],
                self.arr[_sli_head, _sli_tail, :],
            ),
            axis=0,
        ).reshape(-1, 3)
        # printerr(corner_pixels.shape)
        # raise ValueError

        if not self._check_std(corner_pixels):
            raise ValueError("standard variance is too large")

        arr = self.arr
        yi = self._measure(arr, corner_pixels)
        yo = arr.shape[0] - self._measure(arr[::-1, :, :], corner_pixels)

        arr = self.arr.transpose((1, 0, 2))
        xi = self._measure(arr, corner_pixels)
        xo = arr.shape[0] - self._measure(arr[::-1, :, :], corner_pixels)
        self.box = xi, yi, xo, yo

    @classmethod
    def from_file(cls, path, stdmax=STDMAX):
        return cls(Image.open(path), stdmax)

    def crop(self):
        return self.img.crop(self.box)

    def save(self, outpath):
        return self.crop().save(outpath)

    def _check_std(self, pixels):
        if any(list(pixels.std(axis=0) > self.maxstd)):
            return False
        return True

    def _measure(self, arr, corner_pixels):
        """
        :param arr: W x H x 3
        :param corner_pixels: ? x 3
        :return:
        """
        for i in range(arr.shape[0]):
            line = arr[i, :, :]
            line.shape = -1, 3
            pixels = numpy.concatenate((line, corner_pixels), axis=0)
            if not self._check_std(pixels):
                return i
        return 0


def _backup(path):
    px = pathlib.Path(path)
    for i in range(100):
        tag = ".ORIG-{:02}".format(i)
        px_bak = px.with_suffix(tag + px.suffix)
        if not px_bak.exists():
            return px.rename(px_bak)


def margin_crop(path, stdmax, backup=True):
    mdet = MarginDetection.from_file(path, stdmax)
    backup and _backup(path)
    mdet.save(path)


def run(prog=None, args=None):
    desc = "Crop images with homogeneously colored margins "
    pr = argparse.ArgumentParser(prog=prog, description=desc)
    pr.add_argument(
        "-s", "--stdmax", default=STDMAX, type=int, help="maximum standard deviation"
    )
    pr.add_argument(
        "-B", "--no-backup", action="store_true", help="do NOT backup original file"
    )
    pr.add_argument(
        "files", nargs="*", metavar="PATH", help="paths to files to be cropped"
    )
    ns = pr.parse_args(args)
    for path in ns.files:
        try:
            margin_crop(path, ns.stdmax, not ns.no_backup)
        except Exception as e:
            printerr("Failed:", path, "--", e)
            continue



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import os
import re

Pathlike = str | os.PathLike[str]


def parse_colon_kv(text: str):
    result = {}
    for line in text.splitlines():
        if ":" not in line:
            continue
        line = line.strip()
        key, value = line.split(":", 1)
        result[key.rstrip()] = value.lstrip()
    return result


def sanitize_key(s: str):
    # convert to lowercase and remove non-alphanumeric characters except underscores
    s = s.replace(" ", "_")
    s = re.sub(r"\W+", "", s.lower())
    # if the first character is a number, prepend an underscore
    if s and s[0].isdigit():
        s = "_" + s
    return s



#!/usr/bin/env python3
# coding: utf-8
import argparse
import os
import re
import sys
import traceback
from collections import defaultdict

from joker.cast.syntax import printerr
from joker.cast.timedate import sexagesimal_format, sexagesimal_parse
from joker.textmanip.path import proper_filename

from joker.studio.aux.info import MediaInfo
from joker.studio.misc.rename import compute_video_hash


class VideoIdentifier(object):
    version = "vid4_"
    __slots__ = ["duration", "vhash", "w", "h"]

    def __init__(self, duration, vhash, width, height):
        self.duration = duration
        self.vhash = vhash
        self.w = int(width)
        self.h = int(height)

    def __str__(self):
        return "{}-{}x{}".format(self.unikey, self.w, self.h)

    @property
    def unikey(self):
        ds = sexagesimal_format(int(self.duration))
        ds = ds.rjust(3, "0")
        return self.version + "{}_{}".format(ds, self.vhash)

    @classmethod
    def get_regex(cls):
        p = "^" + VideoIdentifier.version
        return re.compile(p + r"([0-9a-yA-Y]{,4})_([0-9A-Z]{,80})-(\d+)x(\d+)")

    @classmethod
    def parse(cls, string):
        mat = cls.get_regex().match(string)
        if mat is None:
            msg = "bad identifier string: {}".format(repr(string))
            raise ValueError(msg)
        ds, fc, w, h = mat.groups()
        return cls(sexagesimal_parse(ds), fc, w, h)

    @classmethod
    def from_name(cls, path):
        xinfo = MediaInfo(path)
        duration = xinfo.get_video_duration()
        vh = compute_video_hash(path)
        v = xinfo.video
        return cls(duration, vh, v.width, v.height)

    @classmethod
    def make_name(cls, path):
        dir_, name = os.path.split(path)
        if name.startswith(cls.version):
            vident = cls.parse(name.split(".")[0])
            return vident, path

        vident = cls.from_name(path)
        new_name = "{}.{}".format(vident, proper_filename(name))
        new_path = os.path.join(dir_, new_name)
        return vident, new_path

    @classmethod
    def rename(cls, path):
        vident, new_path = cls.make_name(path)
        if new_path != path:
            print("new_path:", new_path, file=sys.stderr)
            os.rename(path, new_path)
        return vident, new_path


def p_filter_by_extension(paths):
    extensions = {".mp4", ".ts", ".mkv"}
    _splitext = os.path.splitext
    return [p for p in paths if _splitext(p)[1] in extensions]


def p_filter_by_prefix(paths):
    vp = VideoIdentifier.version
    _split = os.path.split
    return [p for p in paths if _split(p)[1].startswith(vp)]


def keyfunc(pair):
    vi, path = pair
    return vi.w, path.endswith(".mp4")


def show(paths):
    for path in paths:
        try:
            vi = VideoIdentifier.from_name(path)
            print(vi, path, sep="\t")
        except Exception as e:
            # printerr(e)
            traceback.print_exc()
            printerr("bad file:", path)


def add_vident_prefix(paths):
    groups = defaultdict(list)
    for path in p_filter_by_extension(paths):
        try:
            vi, path = VideoIdentifier.rename(path)
        except ValueError:
            traceback.print_exc()
            printerr("bad file:", path)
            continue
        groups[vi.unikey].append((vi, path))

    for pairs in groups.values():
        if len(pairs) < 2:
            continue
        pairs.sort(key=keyfunc, reverse=True)
        print("\n#    ", pairs[0][1])
        for _, path in pairs[1:]:
            print("rm -f", path)


def remove_vident_prefix(paths):
    paths = p_filter_by_extension(paths)
    for path in p_filter_by_prefix(paths):
        dir_, name = os.path.split(path)
        regex = VideoIdentifier.get_regex()
        new_name = regex.sub("", name)
        if new_name.startswith("."):
            new_name = new_name[1:]
        new_path = os.path.join(dir_, new_name)
        if new_name and not os.path.exists(new_path):
            os.rename(path, new_path)
        else:
            printerr("cannot rename:")
            printerr(' from "{}"'.format(path))
            printerr('   to "{}"'.format(new_path))


def run(prog=None, args=None):
    desc = "rename video files by adding or removing video identifier"
    parser = argparse.ArgumentParser(prog=prog, description=desc)

    # mutually exclusive
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-a",
        "--add-prefix",
        action="store_true",
        help="rename with vident prefix",
    )
    group.add_argument(
        "-r",
        "--remove-prefix",
        action="store_true",
        help="rename with vident prefix removed",
    )
    parser.add_argument("paths", metavar="PATH", nargs="+", help="files")
    ns = parser.parse_args(args)
    if ns.add_prefix:
        add_vident_prefix(ns.paths)
    elif ns.remove_prefix:
        remove_vident_prefix(ns.paths)
    else:
        show(ns.paths)


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# coding: utf-8

from joker.studio.aux.utils import rel_shorthand


def vf_crop(width, height, *margins):
    top, right, bottom, left = rel_shorthand(width, height, *margins)
    ow = width - left - right
    oh = height - top - bottom
    return "crop={}:{}:{}:{}".format(ow, oh, left, top)


def vf_subtitle(path, styles):
    if not styles:
        return "subtitles=" + path
    if isinstance(styles, dict):
        styles = styles.items()
    stl = ",".join("{}={}".format(*p) for p in styles)
    return "subtitles={}:force_style='{}'".format(path, stl)


def af_fade(cutin, cutout, fadein, fadeout):
    """
    Genterate FFmpeg audio fade filter string
    https://ffmpeg.org/ffmpeg-filters.html#afade
    :param cutin: (number) position in the original audio where segment starts
    :param cutout: (number) position in the original audio where segment ends
    :param fadein: (number) length of audio where fade-in applied, in sec
    :param fadeout: (number) length of audio where fade-out applied, in sec
    :return: (str) ffmpeg audio filter string
    """
    a = cutin, fadein, cutout - fadeout, fadeout
    return "afade=t=in:st={}:d={},afade=t=out:st={}:d={}".format(*a)


def vf_fade(cutin, cutout, fadein, fadeout, color="black"):
    """
    Genterate FFmpeg video fade filter string
    https://ffmpeg.org/ffmpeg-filters.html#fade
    :param cutin: (number) position in the original video where segment starts
    :param cutout: (number) position in the original video where segment ends
    :param fadein: (number) length of video where fade-in applied, in sec
    :param fadeout: (number) length of video where fade-out applied, in sec
    :param color: (str)
    :return: (str) ffmpeg video filter string
    """
    a = cutin, fadein, color, cutout - fadeout, fadeout, color
    return "fade=t=in:st={}:d={}:c={},fade=t=out:st={}:d={}:c={}".format(*a)



#!/usr/bin/env python3
# coding: utf-8

import argparse
import datetime
import itertools
import os
import pathlib
import re
import shlex
from functools import lru_cache, partial
from subprocess import PIPE

from joker.stream.utils import checksum
from joker.studio.aux import utils
from joker.studio.aux.utils import format_help_section
from joker.studio.ffmpeg.thumb import mkcod_video_thumbnail


@lru_cache(maxsize=64)
def get_xinfo(path):
    from joker.studio.aux.info import MediaInfo

    return MediaInfo(path)


def compute_hash(px, algo="md5"):
    return checksum(px, algo=algo).hexdigest()


def compute_image_hash(px):
    # imagehash is slow to import, 250ms
    import imagehash
    from PIL import Image

    try:
        ih = imagehash.average_hash(Image.open(str(px)))
    except OSError:
        return "NotImage"
    return str(ih).upper()


def compute_video_hash(px, hash_size=4, img_count=4):
    # imagehash is slow to import, 250ms
    import imagehash

    path = str(px)
    xinfo = get_xinfo(path)
    duration = xinfo.get_video_duration()
    params = {
        "tspan": 1.0 * duration / (img_count + 1),
        "count": img_count,
        "size": (160, 90),
    }
    cod = mkcod_video_thumbnail(path, "-", **params)
    with open(os.devnull, "w") as devnull:
        cp = cod.run(quiet=True, stdout=PIPE, stderr=devnull)
    images = utils.split_stream_png(cp.stdout)
    ihs = [imagehash.average_hash(img, hash_size) for img in images]
    return "".join([str(ih) for ih in ihs]).upper()


def get_duration(px):
    parts = ["Dur"]
    dur = int(get_xinfo(px).get_duration())
    dt = datetime.datetime.fromtimestamp(dur)
    if dt.hour:
        parts.append(str(dt.hour).rjust(2, "0") + "h")
    if dt.hour or dt.minute:
        parts.append(str(dt.minute).rjust(2, "0") + "m")
    parts.append(str(dt.second).rjust(2, "0") + "s")
    return "".join(parts)


_lower3 = {
    ".jpeg": ".jpg",
    ".mpeg": ".mpg",
    ".tiff": ".tif",
    ".png_large": ".png",
    ".jpg_large": ".jpg",
    ".bmp_large": ".bmp",
}


def lower3_extension(ext):
    ext = ext.lower()
    return _lower3.get(ext, ext)


def sanitize(px):
    # windows file names disallow:  <>:"/|?* back-slash
    # need to be quoted on unix:    !$&();=@[^`
    # x27 single-quote
    # x5C back-slash
    # x7F delete
    regex = re.compile(r'(^-|[\x00-\x20!$&();=@[^`<>:"/|?*\x27\x5C\x7F]+)')
    name = regex.sub("%", px.name)
    stem, ext = os.path.splitext(name)
    return stem + lower3_extension(ext)


def camel_case(px):
    parts = px.name.split()
    parts = [w[0].upper() + w[1:] for w in parts]
    return "".join(parts)


_known_fields = {
    "IH": compute_image_hash,
    "VH": compute_video_hash,
    "MD5": partial(compute_hash, algo="md5"),
    "SHA1": partial(compute_hash, algo="sha1"),
    "SHA256": partial(compute_hash, algo="sha256"),
    "SHA512": partial(compute_hash, algo="sha512"),
    "WxH": lambda px: "{}x{}".format(*get_xinfo(px).get_size()),
    "DURATION": lambda px: get_duration,
    "EXT": lambda px: px.suffix[1:],
    "EXTL3": lambda px: lower3_extension(px.suffix)[1:],
    "STEM": lambda px: px.stem,
    "NAME": lambda px: px.name,
    "CCNAME": camel_case,
    "SANNAME": sanitize,
}

_variables = {
    "IH": "imagehash.average_hash()",
    "VH": "video hash based on imagehash.average_hash()",
    "MD5": "32 hex digits",
    "SHA1": "40 hex digits",
    "SHA256": "64 hex digits",
    "SHA512": "128 hex digits",
    "WxH": "image or video size",
    "DURATION": "audio or video duration",
    "EXT": "original extension, without a leading dot",
    "EXTL3": "lower3 extension, without a leading dot",
    "STEM": "original file name without extension",
    "NAME": "original file name",
    "CCNAME": "CamelCased file name",
    "SANNAME": "sanitized file name",
}

_safe_presets = {
    "default": "md5-MD5.NAME",
    "md5": "md5-MD5.NAME",
    "sha1": "sha1-SHA1.NAME",
    "ih": "ih-IH.NAME",
    "vh": "vh-VH.NAME",
    "s": "ser-SERIAL.NAME",
    "avatar": "avatar-WxH.NAME",
    "i": "img-WxH.NAME",
    "a": "a-DURATION.NAME",
    "v": "v-WxH-DURATION.NAME",
    "n": "NAME",
}

_risky_presets = {
    "md5%": "md5-MD5.EXT",
    "sha1%": "sha1-SHA1.EXT",
    "ih%": "ih-IH.EXT",
    "vh%": "vh-VH.EXT",
    "s%": "ser-SERIAL.EXT",
    "cc": "CCNAME",
    "san": "SANNAME",
}


def presets_lookup(name):
    return _safe_presets.get(name) or _risky_presets.get(name, name)


class FormulaRenamer(object):
    def __init__(self, formula, clear=False, start=101):
        self._clear = clear
        self._serial = itertools.count(start)
        formula = presets_lookup(formula)
        self._fields = re.split(r"(\W+)", formula)

    @staticmethod
    def clear_preset_prefixes(px):
        _patterns = [
            re.compile(r"^(md5|sha1)-[0-9a-f]{32,40}\."),
            re.compile(r"^(img|avatar|v|a)(-\d+x\d+|-Dur[0-9hms]+){1,2}\."),
            re.compile(r"^(ih|vh)-[0-9A-F]{10,20}."),
        ]
        name = px.name
        for pat in _patterns:
            name = re.sub(pat, "", name)
        return px.with_name(name)

    def make_name(self, px):
        parts = []
        if self._clear:
            px = self.clear_preset_prefixes(px)
        for k in self._fields:
            if k in _known_fields:
                s = _known_fields[k](px)
                parts.append(s)
            elif k == "SERIAL":
                parts.append(str(next(self._serial)))
            else:
                parts.append(k)
        return "".join(parts)

    @staticmethod
    def _case_insen_rename(old, new):
        tag = "__" + os.urandom(4).hex()
        interm = new + tag
        os.rename(old, interm)
        os.rename(interm, new)

    def rename(self, path):
        path = os.path.abspath(path)
        px = pathlib.Path(path)
        name = self.make_name(px)
        new_px = px.with_name(name)
        new_path = str(new_px)
        if path == new_path:
            return
        q = shlex.quote
        print("mv -t", q(path), q(new_path))
        if path.lower() != new_path.lower():
            return px.rename(new_px)
        self._case_insen_rename(path, new_path)


def run(prog=None, args=None):
    desc = "Rename files with a formula"
    s = " (case sensitive)"
    epilog = "\n".join(
        [
            format_help_section("Variables" + s, _variables),
            format_help_section("Safe presets", _safe_presets),
            format_help_section("Risky presets", _risky_presets),
        ]
    )

    pr = argparse.ArgumentParser(
        prog=prog,
        description=desc,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    pr.add_argument(
        "-f",
        "--formula",
        default=_safe_presets["default"],
        help="a name formula, or one of the presets",
    )

    pr.add_argument(
        "-c",
        "--clear",
        action="store_true",
        help="clear previously add tags using presets",
    )

    pr.add_argument(
        "-i",
        "--start",
        default=101,
        type=int,
        help="start number of the SERIAL variable",
    )

    pr.add_argument("files", metavar="PATH", nargs="+")

    ns = pr.parse_args(args)
    fren = FormulaRenamer(ns.formula, ns.clear, ns.start)
    for path in ns.files:
        fren.rename(path)



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import importlib
import json
import re

import setuptools
from volkanic.introspect import find_all_plain_modules
from volkanic.utils import printerr

from joker.studio.environ import GlobalInterface

gi = GlobalInterface()

dotpath_prefixes = [
    "joker.",
    # 'tests.',
]


def test_json_integrity():
    for path in gi.project_dir.rglob("*.json"):
        print("checking json file", path)
        json.load(path.open())


def _check_prefix(path):
    for prefix in dotpath_prefixes:
        if path.startswith(prefix):
            return True
    return False


def test_module_imports():
    pdir = gi.under_project_dir()
    for dotpath in find_all_plain_modules(pdir):
        if _check_prefix(dotpath):
            print("importing", dotpath)
            importlib.import_module(dotpath)


def read_all_py_files():
    pdir = gi.under_project_dir()
    for path in setuptools.findall(pdir):
        if not path.endswith(".py"):
            continue
        for idx, line in enumerate(open(path)):
            line = line.strip()
            if not line:
                continue
            yield path, idx, line


if __name__ == "__main__":
    test_module_imports()
    test_json_integrity()



#!/usr/bin/env python3
# coding: utf-8

import sys
from collections import OrderedDict


def rescale(wsrc, hsrc, w=None, h=None):
    if w is None and h is None:
        w, h = wsrc, hsrc
    elif w is None:
        w = 1.0 * h * wsrc / hsrc
    elif h is None:
        h = 1.0 * w * hsrc / wsrc
    return int(w / 2.0) * 2, int(h / 2.0) * 2


def hexformat(nums):
    return "".join((format(n, "X") for n in nums))


def css_shorthand(*values):
    """css 4-edge shorthand"""
    n = len(values)
    if n <= 1:
        top, right, bottom, left = (list(values) or [0]) * 4
    elif n == 2:
        top = bottom = values[0]
        right = left = values[1]
    elif n == 3:
        top, right, bottom = values
        left = right
    elif n:
        top, right, bottom, left = values
    else:
        raise ValueError("too many values for margins")
    return top, right, bottom, left


# relative_shorthand?
def rel_shorthand(width, height, *values):
    """css 4-edge shorthand with fraction support"""
    top, right, bottom, left = css_shorthand(*values)
    top = int(top if top >= 1.0 else height * top)
    left = int(left if left >= 1.0 else width * left)
    right = int(right if right >= 1.0 else width * right)
    bottom = int(bottom if bottom >= 1.0 else height * bottom)
    return top, right, bottom, left


def split_by_header(s, h):
    return [h + x for x in s.split(h) if x]


def split_stream_png(binstr):
    import io
    from PIL import Image

    h = b"\x89PNG\r\n\x1a\n"
    ims = split_by_header(binstr, h)
    return [Image.open(io.BytesIO(x)) for x in ims]


# TODO: move to j.tmanip
def _kvfmt(k, v, n, colon=":"):
    k = (k + colon).ljust(n)
    return "{}{}\n".format(k, v)


# TODO: move to j.tmanip
def format_dictionary(d, bullet="*", colon=":"):
    n = max(len(k) for k in d) + 3
    parts = [bullet + _kvfmt(k, v, n, colon) for k, v in d.items()]
    return "".join(parts)


# TODO: move to j.tmanip
def format_help_section(title, content):
    return title + ":\n" + format_dictionary(content, "  ", "") + "\n\n"


def add_dry_option(argparser):
    argparser.add_argument(
        "--dry", action="store_true", help="print ffmpeg command but do not execute it"
    )


class CommandOptionDict(OrderedDict):
    def __init__(self, *args, **kwargs):
        super(CommandOptionDict, self).__init__(*args, **kwargs)
        self.pargs = list()
        self.executable = ""

    def as_args(self):
        parts = [self.executable]
        for k, v in self.items():
            if v is None:
                continue
            parts.append("-{}".format(k))
            if isinstance(v, (tuple, list)):
                parts.extend(v)
            else:
                parts.append(v)
        parts.extend(self.pargs)
        return list(map(str, parts))

    def __str__(self):
        import shlex

        args = [shlex.quote(s) for s in self.as_args()]
        return " ".join(args)

    def run(self, dry=False, quiet=False, **kwargs):
        if not quiet:
            print(self, file=sys.stderr)
        if not dry:
            import subprocess

            return subprocess.run(self.as_args(), **kwargs)

    def __call__(self, executable, *pargs):
        # for convenience
        self.executable = executable
        self.pargs = list(pargs)
        return self



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import os
import subprocess
import sys
from pathlib import Path

from joker.studio.utils import parse_colon_kv, sanitize_key, Pathlike


def run_tesseract_osd_only(path: Pathlike) -> dict:
    cmd = [
        "tesseract",
        path,
        "-",
        "--psm",
        "0",
    ]
    subp = subprocess.run(cmd, capture_output=True)
    s = subp.stdout.decode("utf-8")
    return {sanitize_key(k): v for k, v in parse_colon_kv(s).items()}


def infer_rotation_degrees(path: Pathlike) -> int:
    osd_dict = run_tesseract_osd_only(path)
    return int(osd_dict.get("rotate", 0))


def _get_output_path(path: Pathlike):
    path = Path(path).absolute()
    infix = os.urandom(8).hex()
    return path.with_stem(f"{path.stem}-{infix}")


def rotate_image(path: Pathlike) -> Path | None:
    degrees = infer_rotation_degrees(path)
    if not degrees:
        return None
    outpath = _get_output_path(path)
    cmd = [
        "magick",
        path,
        "-rotate",
        degrees,
        outpath,
    ]
    subprocess.run([str(s) for s in cmd])
    return outpath


def main(_prog, args: list[str]):
    for path in args:
        rotate_image(path)


if __name__ == "__main__":
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/env python3
# coding: utf-8

import argparse
import math
import pathlib

from joker.studio.aux import utils
from joker.studio.aux.info import MediaInfo


def _labelfunc(startpos, duration, step):
    m, s = divmod(startpos, 60)
    h, m = divmod(m, 60)
    hh = "{:02}".format(h) if duration >= 3600 else ""
    mm = "{:02}".format(m)
    ss = "{:02}".format(s) if step % 60 else "M"
    return hh + mm + ss


class MediaSplitter(object):
    def __init__(self, path):
        self.px = pathlib.Path(path)
        self.path = path
        self.xinfo = MediaInfo(path)
        self.duration = math.ceil(self.xinfo.get_duration())

    def _fmt_label(self, startpos, precise=True):
        startpos = int(startpos)
        m, s = divmod(startpos, 60)
        h, m = divmod(m, 60)
        hh = "{:02}".format(h) if self.duration >= 3600 else ""
        mm = "{:02}".format(m)
        ss = "{:02}".format(s) if precise else "M"
        return hh + mm + ss

    def split(self, position_pairs):
        position_pairs = list(position_pairs)
        precise = any(pair[0] % 60 for pair in position_pairs)
        for start, length in position_pairs:
            cod = utils.CommandOptionDict(
                [
                    ("i", self.px),
                    ("ss", start),
                    ("t", length),
                    ("c:v", "copy"),
                    ("c:a", "copy"),
                ]
            )
            label = ".SPLIT-" + self._fmt_label(start, precise)
            px_out = self.px.with_suffix(label + self.px.suffix)
            yield cod("ffmpeg", px_out)

    def uniform_split(self, step):
        # step = duration / num if num else ns.step
        step = math.ceil(step)
        start_positions = range(0, self.duration, step)
        return self.split((p, step) for p in start_positions)

    def custom_split(self, positions):
        positions = list(positions)
        positions.sort()
        if positions[0] != 0:
            positions.insert(0, 0)
        if positions[-1] < self.duration:
            positions.append(self.duration)
        pairs = [(a, b - a) for a, b in zip(positions[:-1], positions[1:])]
        return self.split(pairs)

    def silence_split(self):
        from joker.studio.alpha.audio import AudioEnergySeries

        aes = AudioEnergySeries.from_file(self.path)
        positions = aes.find_silences()
        return self.custom_split(positions)


def run(prog=None, args=None):
    desc = "Split a video uniformly or at specified positions"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    utils.add_dry_option(parser)

    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-n", "--num", type=int, metavar="INT", help="number of segments to split into"
    )
    group.add_argument(
        "-s",
        "--step",
        type=int,
        default=240,
        metavar="INT",
        help="length of each segment, in second",
    )
    group.add_argument(
        "-p",
        "--positions",
        type=int,
        nargs="+",
        metavar="INT",
        help="positions of splits",
    )
    group.add_argument("-q", "--silence", action="store_true", help="split on silences")

    parser.add_argument("path", help="a video or audio file")
    ns = parser.parse_args(args)
    mspl = MediaSplitter(ns.path)
    if ns.silence:
        cods = mspl.silence_split()
    elif ns.positions:
        cods = mspl.custom_split(ns.positions)
    else:
        step = mspl.duration / ns.num if ns.num else ns.step
        cods = mspl.uniform_split(step)
    for cod in cods:
        cod.run(ns.dry)


if __name__ == "__main__":
    run()



#!/usr/bin/env python3
# coding: utf-8

import argparse
import pathlib

from joker.studio.aux import utils
from joker.studio.ffmpeg.filters import vf_subtitle


def mkcod_subtitle(path, subpath, styles, outpath):
    cod = utils.CommandOptionDict(
        [
            ("i", path),
            ("c:a", "copy"),
            ("vf", vf_subtitle(subpath, styles)),
        ]
    )
    return cod("ffmpeg", outpath)


_ref_styles = {
    "Alignment": None,
    "Angle": None,
    "BackColour": None,
    "Bold": None,
    "BorderStyle": None,
    "Encoding": None,
    "Fontname": None,
    "Fontsize": None,
    "Italic": None,
    "MarginL": None,
    "MarginR": None,
    "MarginV": None,
    "Name": None,
    "Outline": None,
    "OutlineColour": None,
    "PrimaryColour": None,
    "ScaleX": None,
    "ScaleY": None,
    "SecondaryColour": None,
    "Shadow": None,
    "Strikeout": None,
    "Underline": None,
}

_styles = [
    ("Fontname", "Khmer MN"),
    ("Fontsize", 46),
    ("BackColour", "&HA0000000"),
    ("BorderStyle", 4),
    # ('PrimaryColour', '&HADAEC4'),
    ("OutlineColour", "&H800000"),
]


def run(prog=None, args=None):
    desc = "Burn subtitle into a video"
    parser = argparse.ArgumentParser(prog=prog, description=desc)
    utils.add_dry_option(parser)
    parser.add_argument("-s", "--sub", metavar="PATH", help="an SRT file")
    parser.add_argument("path", metavar="PATH", help="a video file")
    ns = parser.parse_args(args)
    px = pathlib.Path(ns.path)
    subpath = ns.sub or px.with_suffix(".srt")
    outpath = px.with_suffix(".wSub" + px.suffix)
    cod = mkcod_subtitle(ns.path, subpath, _styles, outpath)
    cod.run(ns.dry)


if __name__ == "__main__":
    run()
